@@ -15,10 +15,13 @@ from json.decoder import JSONDecodeError
 import urllib3
 from urllib3.connection import match_hostname as urllib3_match_hostname
 from urllib3.util.ssl_ import ssl_wrap_socket as urllib3_ssl_wrap_socket
-from urllib3.util.ssl_ import wrap_socket as urllib3_wrap_socket
+
+try:
+    from urllib3.util.ssl_ import wrap_socket as urllib3_wrap_socket
+except ImportError:
+    urllib3_wrap_socket = None
 
 from .compat import basestring, byte_type, decode_from_bytes, encode_to_bytes, text_type
-from .exceptions import StrictMocketException
 from .utils import (
     SSL_PROTOCOL,
     MocketMode,
@@ -45,6 +48,13 @@ try:  # pragma: no cover
 except ImportError:
     pyopenssl_override = False
 
+try:  # pragma: no cover
+    from aiohttp import TCPConnector
+
+    aiohttp_make_ssl_context_cache_clear = TCPConnector._make_ssl_context.cache_clear
+except (ImportError, AttributeError):
+    aiohttp_make_ssl_context_cache_clear = None
+
 
 true_socket = socket.socket
 true_create_connection = socket.create_connection
@@ -52,7 +62,9 @@ true_gethostbyname = socket.gethostbyname
 true_gethostname = socket.gethostname
 true_getaddrinfo = socket.getaddrinfo
 true_socketpair = socket.socketpair
-true_ssl_wrap_socket = ssl.wrap_socket
+true_ssl_wrap_socket = getattr(
+    ssl, "wrap_socket", None
+)  # in Py3.12 it's only under SSLContext
 true_ssl_socket = ssl.SSLSocket
 true_ssl_context = ssl.SSLContext
 true_inet_pton = socket.inet_pton
@@ -68,6 +80,7 @@ class SuperFakeSSLContext:
         def __set__(self, *args):
             pass
 
+    minimum_version = FakeSetter()
     options = FakeSetter()
     verify_mode = FakeSetter(ssl.CERT_NONE)
 
@@ -78,6 +91,7 @@ class FakeSSLContext(SuperFakeSSLContext):
         "load_verify_locations",
         "set_alpn_protocols",
         "set_ciphers",
+        "set_default_verify_paths",
     )
     sock = None
     post_handshake_auth = None
@@ -164,7 +178,7 @@ class MocketSocket:
     _secure_socket = False
 
     def __init__(
-        self, family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0, *args, **kwargs
+        self, family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0, **kwargs
     ):
         self.true_socket = true_socket(family, type, proto)
         self._buflen = 65536
@@ -173,6 +187,8 @@ class MocketSocket:
         self.type = int(type)
         self.proto = int(proto)
         self._truesocket_recording_dir = None
+        self._did_handshake = False
+        self._sent_non_empty_bytes = False
         self.kwargs = kwargs
 
     def __str__(self):
@@ -211,7 +227,7 @@ class MocketSocket:
         return socket.SOCK_STREAM
 
     def do_handshake(self):
-        pass
+        self._did_handshake = True
 
     def getpeername(self):
         return self._address
@@ -219,6 +235,9 @@ class MocketSocket:
     def setblocking(self, block):
         self.settimeout(None) if block else self.settimeout(0.0)
 
+    def getblocking(self):
+        return self.gettimeout() is None
+
     def getsockname(self):
         return socket.gethostbyname(self._address[0]), self._address[1]
 
@@ -250,6 +269,8 @@ class MocketSocket:
 
     @staticmethod
     def fileno():
+        if Mocket.r_fd is not None:
+            return Mocket.r_fd
         Mocket.r_fd, Mocket.w_fd = os.pipe()
         return Mocket.r_fd
 
@@ -285,10 +306,21 @@ class MocketSocket:
             self.fd.seek(0)
 
     def read(self, buffersize):
-        return self.fd.read(buffersize)
+        rv = self.fd.read(buffersize)
+        if rv:
+            self._sent_non_empty_bytes = True
+        if self._did_handshake and not self._sent_non_empty_bytes:
+            raise ssl.SSLWantReadError("The operation did not complete (read)")
+        return rv
 
     def recv_into(self, buffer, buffersize=None, flags=None):
-        return buffer.write(self.read(buffersize))
+        if hasattr(buffer, "write"):
+            return buffer.write(self.read(buffersize))
+        # buffer is a memoryview
+        data = self.read(buffersize)
+        if data:
+            buffer[: len(data)] = data
+        return len(data)
 
     def recv(self, buffersize, flags=None):
         if Mocket.r_fd and Mocket.w_fd:
@@ -303,8 +335,8 @@ class MocketSocket:
         raise exc
 
     def true_sendall(self, data, *args, **kwargs):
-        if MocketMode().STRICT:
-            raise StrictMocketException("Mocket tried to use the real `socket` module.")
+        if not MocketMode().is_allowed((self._host, self._port)):
+            MocketMode.raise_not_allowed()
 
         req = decode_from_bytes(data)
         # make request unique again
@@ -349,7 +381,7 @@ class MocketSocket:
             encoded_response = hexload(response_dict["response"])
         # if not available, call the real sendall
         except KeyError:
-            host, port = Mocket._address
+            host, port = self._host, self._port
             host = true_gethostbyname(host)
 
             if isinstance(self.true_socket, true_socket) and self._secure_socket:
@@ -380,7 +412,6 @@ class MocketSocket:
 
             # dump the resulting dictionary to a JSON file
             if Mocket.get_truesocket_recording_dir():
-
                 # update the dictionary with request and response lines
                 response_dict["request"] = req
                 response_dict["response"] = hexdump(encoded_response)
@@ -449,8 +480,12 @@ class Mocket:
 
     @classmethod
     def reset(cls):
-        cls.r_fd = None
-        cls.w_fd = None
+        if cls.r_fd is not None:
+            os.close(cls.r_fd)
+            cls.r_fd = None
+        if cls.w_fd is not None:
+            os.close(cls.w_fd)
+            cls.w_fd = None
         cls._entries = collections.defaultdict(list)
         cls._requests = []
 
@@ -509,15 +544,20 @@ class Mocket:
         urllib3.util.ssl_.ssl_wrap_socket = urllib3.util.ssl_.__dict__[
             "ssl_wrap_socket"
         ] = FakeSSLContext.wrap_socket
+        urllib3.util.ssl_wrap_socket = urllib3.util.__dict__[
+            "ssl_wrap_socket"
+        ] = FakeSSLContext.wrap_socket
         urllib3.connection.ssl_wrap_socket = urllib3.connection.__dict__[
             "ssl_wrap_socket"
         ] = FakeSSLContext.wrap_socket
         urllib3.connection.match_hostname = urllib3.connection.__dict__[
             "match_hostname"
-        ] = lambda cert, hostname: None
+        ] = lambda *args: None
         if pyopenssl_override:  # pragma: no cover
             # Take out the pyopenssl version - use the default implementation
             extract_from_urllib3()
+        if aiohttp_make_ssl_context_cache_clear:  # pragma: no cover
+            aiohttp_make_ssl_context_cache_clear()
 
     @staticmethod
     def disable():
@@ -531,7 +571,8 @@ class Mocket:
         socket.gethostbyname = socket.__dict__["gethostbyname"] = true_gethostbyname
         socket.getaddrinfo = socket.__dict__["getaddrinfo"] = true_getaddrinfo
         socket.socketpair = socket.__dict__["socketpair"] = true_socketpair
-        ssl.wrap_socket = ssl.__dict__["wrap_socket"] = true_ssl_wrap_socket
+        if true_ssl_wrap_socket:
+            ssl.wrap_socket = ssl.__dict__["wrap_socket"] = true_ssl_wrap_socket
         ssl.SSLContext = ssl.__dict__["SSLContext"] = true_ssl_context
         socket.inet_pton = socket.__dict__["inet_pton"] = true_inet_pton
         urllib3.util.ssl_.wrap_socket = urllib3.util.ssl_.__dict__[
@@ -540,6 +581,9 @@ class Mocket:
         urllib3.util.ssl_.ssl_wrap_socket = urllib3.util.ssl_.__dict__[
             "ssl_wrap_socket"
         ] = true_urllib3_ssl_wrap_socket
+        urllib3.util.ssl_wrap_socket = urllib3.util.__dict__[
+            "ssl_wrap_socket"
+        ] = true_urllib3_ssl_wrap_socket
         urllib3.connection.ssl_wrap_socket = urllib3.connection.__dict__[
             "ssl_wrap_socket"
         ] = true_urllib3_ssl_wrap_socket
@@ -550,6 +594,8 @@ class Mocket:
         if pyopenssl_override:  # pragma: no cover
             # Put the pyopenssl version back in place
             inject_into_urllib3()
+        if aiohttp_make_ssl_context_cache_clear:  # pragma: no cover
+            aiohttp_make_ssl_context_cache_clear()
 
     @classmethod
     def get_namespace(cls):
@@ -592,14 +638,15 @@ class MocketEntry:
         else:
             self.responses = []
             for r in responses:
-                if isinstance(r, BaseException):
-                    pass
-                elif not getattr(r, "data", False):
+                if not isinstance(r, BaseException) and not getattr(r, "data", False):
                     if isinstance(r, text_type):
                         r = encode_to_bytes(r)
                     r = self.response_cls(r)
                 self.responses.append(r)
 
+    def __repr__(self):
+        return "{}(location={})".format(self.__class__.__name__, self.location)
+
     @staticmethod
     def can_handle(data):
         return True
@@ -628,11 +675,18 @@ class Mocketizer:
         namespace=None,
         truesocket_recording_dir=None,
         strict_mode=False,
+        strict_mode_allowed=None,
     ):
         self.instance = instance
         self.truesocket_recording_dir = truesocket_recording_dir
         self.namespace = namespace or text_type(id(self))
         MocketMode().STRICT = strict_mode
+        if strict_mode:
+            MocketMode().STRICT_ALLOWED = strict_mode_allowed or []
+        elif strict_mode_allowed:
+            raise ValueError(
+                "Allowed locations are only accepted when STRICT mode is active."
+            )
 
     def enter(self):
         Mocket.enable(
@@ -667,7 +721,7 @@ class Mocketizer:
             method()
 
     @staticmethod
-    def factory(test, truesocket_recording_dir, strict_mode, args):
+    def factory(test, truesocket_recording_dir, strict_mode, strict_mode_allowed, args):
         instance = args[0] if args else None
         namespace = None
         if truesocket_recording_dir:
@@ -684,11 +738,21 @@ class Mocketizer:
             namespace=namespace,
             truesocket_recording_dir=truesocket_recording_dir,
             strict_mode=strict_mode,
+            strict_mode_allowed=strict_mode_allowed,
         )
 
 
-def wrapper(test, truesocket_recording_dir=None, strict_mode=False, *args, **kwargs):
-    with Mocketizer.factory(test, truesocket_recording_dir, strict_mode, args):
+def wrapper(
+    test,
+    truesocket_recording_dir=None,
+    strict_mode=False,
+    strict_mode_allowed=None,
+    *args,
+    **kwargs,
+):
+    with Mocketizer.factory(
+        test, truesocket_recording_dir, strict_mode, strict_mode_allowed, args
+    ):
         return test(*args, **kwargs)
 
 
